/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*

	This file contains two classes:
	- one struct that contains the state of a blue window slider
	- one class that encapsulates the entire blue window state (bw cursor + all sliders)


 */


#ifndef ARPIBlueWindowPreset_h
#define ARPIBlueWindowPreset_h

#ifndef ARPIBlueWindowPreset_h
#include "ARPIBlueWindowPreset.h"
#endif //ARPIBlueWindowPreset_h

#ifndef ZzTypes_h
#include "../../../Zz/ZzTypes.h"
#endif //ZzTypes_h

#define ARPIBlueWindowPresetMaxNbParams 32
#define ARPIBlueWindowPresetMaxNbSources 3
#define ARPIBlueWindowPresetNameLength 64


// macros for smooth preset initialization

#define PRESET_NEXT				presetPtr++;
#define PRESET_NAME(x)			presetPtr->SetName( x );
#define PRESET_SOURCE(x)		presetPtr->AddSource((float)(x));
#define PRESET_PARAM(val, min, max, source)								\
{																		\
	ARPIBlueWindowParamState paramstate = 								\
		{(float)(val), (float)(min), (float)(max), (SInt32)(source)};	\
	presetPtr->AddParam(&paramstate);									\
}
	


#ifndef AUDIO_UNITS_BUILD
class ArmeArchiveHandler;
#endif //#ifndef AUDIO_UNITS_BUILD


/* 
important note: it is not true that MinValue < Maxvalue, so the name is chosen a bit
awkward. this is to have reverse ramps (crossfades etc...)
*/

struct ARPIBlueWindowParamState
{
	float Value;		// value of the slider, or float encodes int index of an optionbox
	float MinValue;		// left bound of the slider
	float MaxValue;		// right bound of the slider, or float encoded int nbOptions
	SInt32 Source;		// blue window source parameter for this slider 
						// 0 == none, 1 == x, 2 == y, 3 == z, (later: 100 = midi, etc)
#ifndef AUDIO_UNITS_BUILD
	// archive stuff
	void	ReadFromArchive(ArmeArchiveHandler* unarchiver);
	void	WriteToArchive(ArmeArchiveHandler* archiver) const;
#endif //#ifndef AUDIO_UNITS_BUILD
};



class ARPIBlueWindowPreset
{
public:

	// init stuff

	ARPIBlueWindowPreset();
	~ARPIBlueWindowPreset();
	
	// archive stuff
#ifndef AUDIO_UNITS_BUILD
	void	ReadFromArchive(ArmeArchiveHandler* unarchiver);
	void	WriteToArchive(ArmeArchiveHandler* archiver) const;
#endif //#ifnef AUDIO_UNITS_BUILD

	void AddSource(float sourceVal);
	void AddSourceArray(float *sourceArray, SInt32 nbElements);
	void AddParam(ARPIBlueWindowParamState* paramState);
	void AddParamArray(ARPIBlueWindowParamState* paramState, SInt32 nbElements);
	
	// init / re-init stuff
	void SetNbParams(SInt32 nbParams);
	void SetNbSources(SInt32 nbSources);
	void SetName(const char* name);
	void SetSourceValue(SInt32 source, float value);
	void SetParamSource(SInt32 param, SInt32 source);
	void SetParamValue(SInt32 param, float value);
	void SetParamMinValue(SInt32 param, float minValue);
	void SetParamMaxValue(SInt32 param, float maxValue);
	
	// query stuff
	
	SInt32 		GetNbParams();
	SInt32 		GetNbSources();
	const char*	GetName() const;
	float 		GetSourceValue(SInt32 source);
	SInt32 		GetParamSource(SInt32 param);
	float 		GetParamValue(SInt32 param);
	float 		GetParamMinValue(SInt32 param);
	float 		GetParamMaxValue(SInt32 param);

private:

	void UpdateNbParams(SInt32 nbParams);
	void UpdateNbSources(SInt32 nbSources);

	char						fName[ARPIBlueWindowPresetNameLength];
	ARPIBlueWindowParamState	fParam[ARPIBlueWindowPresetMaxNbParams];
	float						fSource[ARPIBlueWindowPresetMaxNbSources];

	SInt32 fNbParams;			
	SInt32 fNbSources;

};










#endif // ARPIBlueWindowPreset_h

